sessiontest

package
v1.6.2 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 13, 2026 License: Apache-2.0 Imports: 3 Imported by: 0

Documentation

Overview

Package sessiontest provides test helpers for the session package.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func MockLocal

func MockLocal() *session.Session

MockLocal creates a mock session for local development without authentication. The mock user has ID "local" and no GitHub access token.

Types

This section is empty.

Jump to